/*
 * N.I.C.H. Monaic - The 2 channel bbs viewer
 * 
 * Copyright (C) 2001 Free Software Foundation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */

/*
 * See the AUTHORS file for a list of people on the Monaic Team. See
 * the ChangeLog files for a list of changes. These files are distributed
 * with Monaic at http://monaic.sourceforge.net/
 */

#include "bbstable.h"

#include "http.h"

#define TABLEURL "http://www6.ocn.ne.jp/~mirv/bbstable2.html"

typedef struct _MonaBBSTable {
  gchar* category;
  gchar* url;
  gchar* name;
} MonaBBSTable;

gint mona_strncmp(gchar* s1, gchar* s2) {
  return strncmp(s1, s2, strlen(s2));
}

gint mona_strncasecmp(gchar* s1, gchar* s2) {
  return strncasecmp(s1, s2, strlen(s2));
}

void on_table_menu_activate(GtkWidget* widget, gpointer data) {
  MonaicInfo* monaicinfo;

  monaicinfo = monaic_get_monaicinfo();

  get_board_info((gchar*)data, monaicinfo->boardinfo);
  monaic_gui_thread_update();
}

GSList* get_bbs_table() {
  mona_http* http = NULL;
  gchar* rawhtml = NULL;
  gchar* euchtml = NULL;
  GSList* tablelist = NULL;
  gchar* cursor = NULL;
  gchar* buffer = NULL;
  guchar* category = NULL;
  gint i = 0;
  gint kcc_ret = 0;

  http = mona_http_get(TABLEURL);
  g_assert(http != NULL);
  euchtml = sjis2euc(http->dat, http->size);
  g_assert(euchtml != NULL);

  tablelist = g_slist_alloc();
  for(cursor = euchtml; *cursor != '\0'; cursor++ ) {
    while(*cursor != '<' && mona_strncmp(cursor, "<B>") != 0
          && *cursor != '\0' ) cursor++;
    if( mona_strncmp(cursor, "<B>") == 0 ) {
      buffer = (gchar*)strstr(cursor, "</B>");
      g_assert(buffer != NULL);
      cursor += strlen("<B>");
      if( category != NULL ) g_free(category);
      g_assert(buffer-cursor > 0);
      category = g_strndup(cursor, buffer-cursor);
      cursor = buffer;
    } else if( mona_strncasecmp(cursor, "<A HREF=http://") == 0 ) {
      MonaBBSTable* newtable = g_new0(MonaBBSTable, 1);
      buffer = (gchar*)strchr(cursor, '>');
      cursor += strlen("<A HREF=");
      g_assert(buffer-cursor > 0);
      newtable->url = g_strndup(cursor, buffer-cursor);

      if( strstr(newtable->url, "www.2ch.net")
          || !strstr(newtable->url, "2ch.net")
          || strstr(newtable->url, "TARGET=_blank") ) {
        g_free(newtable->url);
        g_free(newtable);
        continue;
      }

      cursor = buffer++;
      while( *cursor != '<' ) cursor++;
      if( mona_strncasecmp(cursor, "</A>") ) {
        g_print("Broken HTML\n");
        break;
      } else {
        g_assert(cursor-buffer > 0);
        newtable->name = g_strndup(buffer, cursor-buffer);
      }
      tablelist = g_slist_append(tablelist, newtable);
      cursor += strlen("</A>");
      if( category != NULL )
        newtable->category = g_strdup(category);
      continue;
    }
  }

  if( category != NULL ) g_free(category);

/*
  for(i=1; i<g_slist_length(tablelist); i++) {
    MonaBBSTable *bcursor = (MonaBBSTable*)g_slist_nth_data(tablelist, i);
    g_print("%d category: %s\n", i, bcursor->category);
    g_print("%d url: %s\n", i, bcursor->url);
    g_print("%d name: %s\n", i, bcursor->name);
  }
*/
  return tablelist;
}

void add_bbs_table(GtkMenuItem* menuitem) {
  GtkWidget* submenu = NULL;
  GtkWidget* item = NULL;
  GtkWidget* catmenu = NULL;
  GSList* tablelist = get_bbs_table();
  gint i = 0;
  MonaBBSTable* cursor = NULL;
  guchar* category = NULL;

  g_return_if_fail(GTK_IS_MENU_ITEM(menuitem));
  g_return_if_fail(tablelist != NULL);
  g_return_if_fail(g_slist_length(tablelist) > 1);

  submenu = gtk_menu_new();

  for(i=1; i<g_slist_length(tablelist); i++) {
    cursor = (MonaBBSTable*)g_slist_nth_data(tablelist, i);
    g_assert(cursor != NULL);
    g_assert(cursor->category != NULL);
    g_assert(*(cursor->category) != '\0');

    if( category == NULL || strcmp(category, cursor->category) != 0 ) {
      item = gtk_menu_item_new_with_label(cursor->category);
      gtk_menu_append(GTK_MENU(submenu), item);
      catmenu = gtk_menu_new();
      gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), catmenu);
      if( category != NULL )
        g_free(category);
      category = g_strdup(cursor->category);
    }

    item = gtk_menu_item_new_with_label(cursor->name);
    gtk_signal_connect(GTK_OBJECT(item), "activate",
                       (GtkSignalFunc)on_table_menu_activate,
                       g_strdup(cursor->url));
    gtk_menu_append(GTK_MENU(catmenu), item);
  }

  gtk_menu_item_set_submenu(menuitem, submenu);

  /* Free the garbages */
  for(i=1; i<g_slist_length(tablelist); i++) {
    cursor = (MonaBBSTable*)g_slist_nth_data(tablelist, i);
    if( cursor->category != NULL )
      g_free(cursor->category);
    if( cursor->name != NULL )
      g_free(cursor->name);
    if( cursor->url != NULL)
      g_free(cursor->url);
    g_free(cursor);
  }
  g_slist_free(tablelist);

  if( category != NULL )
    g_free(category);
}
