/*
 * N.I.C.H. Monaic - The 2 channel bbs viewer
 * 
 * Copyright (C) 2001 Free Software Foundation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */

/*
 * See the AUTHORS file for a list of people on the Monaic Team. See
 * the ChangeLog files for a list of changes. These files are distributed
 * with Monaic at http://monaic.sourceforge.net/
 */

#include "http.h"
#include <stdio.h>
#include <ghttp.h>
#include "kcc.h"

gchar* sjis2euc(gchar* inbuf, gint isize) {
  gchar* result = g_new0(gchar, isize*2);
  gint checked = 0;

  if( inbuf == NULL ) return NULL;

  checked = KCC_check(inbuf, 0);

  if( checked == 0 ) { /* ASCII */
    g_free(result);
    return inbuf;
  } else if( checked == 0x10 ) { /* EUC */
    g_free(result);
    return inbuf;
  } else if( checked == 0x40 ) {
    if ( -1 == KCC_filter(result, "EUC", inbuf, "SJIS", 0, 0, 0) ) {
      fprintf(stderr, "libkcc convert error.\n");
      return inbuf;
    }
  } else {
    fprintf(stderr, "WARNING: inbuf is not SJIS: 0x%02x\n", checked);
/*
    fprintf(stderr, "==> %s\n", inbuf);
    fprintf(stderr, "==> %d\n", isize);
*/
    return inbuf;
  }
  return result;
}

mona_http* mona_http_get(gchar* url) {
  mona_http* result = NULL;
  ghttp_request* request = NULL;

  g_return_val_if_fail(url != NULL, NULL);

  result = g_new0(mona_http, 1);
  request = ghttp_request_new();
  ghttp_set_uri(request, url);
  ghttp_set_header(request, http_hdr_Connection, "close");
  ghttp_prepare(request);
  ghttp_process(request);

  result->size = ghttp_get_body_len(request);
  result->dat = g_strndup(ghttp_get_body(request), result->size);

  ghttp_request_destroy(request);
  
  return result;
}

void mona_url2stream(gchar* url, FILE* stream) {
  ghttp_request* request = NULL;

  g_return_if_fail(url != NULL);
  g_return_if_fail(stream != NULL);

  request = ghttp_request_new();
  ghttp_set_uri(request, url);
  ghttp_set_header(request, http_hdr_Connection, "close");
  ghttp_prepare(request);
  ghttp_process(request);

  fwrite(ghttp_get_body(request), ghttp_get_body_len(request), 1, stream);
  fflush(stream);
  ghttp_request_destroy(request);
}

gchar** mona_url2euc(gchar* url) {
  gchar** result = NULL;
  gchar*  cursor = NULL;
  gchar*  tmpstr = NULL;
  mona_http* monahttp = NULL;
  gint i = 0;

  g_return_val_if_fail(url != NULL, NULL);

  monahttp = mona_http_get(url);
  if( !monahttp ) return NULL;

  result = g_strsplit(monahttp->dat, "\n", -1);
  if( !result ) return NULL;

  cursor = *result;
  while( cursor != NULL ) {
    tmpstr = sjis2euc(cursor, strlen(cursor));
    if( !cursor ) g_free(cursor);
    result[i] = tmpstr;
    cursor = result[++i];
  }

  return result;
}
