/*
 * N.I.C.H. Monaic - The 2 channel bbs viewer
 * 
 * Copyright (C) 2001 Free Software Foundation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */

/*
 * See the AUTHORS file for a list of people on the Monaic Team. See
 * the ChangeLog files for a list of changes. These files are distributed
 * with Monaic at http://monaic.sourceforge.net/
 */

#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <ghttp.h>
#include <iconv.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include "kcc.h"
#include "http.h"
#include "bbstable.h"
#include "monatext.h"

#include "monaic.h"

extern int errno;

#define nich_strncmp(buf, str) strncmp(buf, str, strlen(str))

//#define BASEURL "http://aa.2ch.net/mona"
//#define BASEURL "http://life2.2ch.net/credit"
#define BASEURL "http://pc.2ch.net/linux"
#define DELIMIT "<>"
/*
#define BASEURL "http://piza2.2ch.net/mona"
#define DELIMIT "<>"
*/

typedef struct _DotPath {
  gchar* dir;
  gchar* pic;
} DotPath;

DotPath* dotpath = NULL;
static MonaicInfo* monaicinfo = NULL;

GtkWidget* hpaned = NULL;
GtkWidget* picarea = NULL;
GtkWidget* picarea_parent = NULL;

void monaic_quit();
void on_title_pic_toggled(gpointer data, gpointer action, GtkWidget* widget);
void on_picarea_expose(GtkWidget* widget, GdkEventExpose *event, gpointer data);

static GtkItemFactoryEntry menu_items[] =
{
  {"/ե(_F)", NULL, NULL, 0, "<Branch>"},
  {"/ե(_F)/(_N)", NULL, NULL, 0, NULL},
  {"/ե(_F)/(_O)", NULL, NULL, 0, NULL},
  {"/ե(_F)/¸(_S)", NULL, NULL, 0, NULL},
  {"/ե(_F)/̾Ѥ¸(_A)", NULL, NULL, 0, NULL},
  {"/ե(_F)/---", NULL, NULL, 0, "<Separator>"},
  {"/ե(_F)/λ(_Q)", "<control>Q", monaic_quit, 0, NULL},

  {"/ɽ(_V)", NULL, NULL, 0, "<Branch>"},
  {"/ɽ(_V)/ɤ߹(_R)", "<control>R", NULL, 0, NULL},
  {"/ɽ(_V)/ĥХʡ(_P)", NULL, on_title_pic_toggled, 0, "<ToggleItem>"},

  {"/å(_T)", NULL, NULL, 0, "<Branch>"},

  {"/(_S)", NULL, NULL, 0, "<Branch>"},
  {"/(_S)/(_P)", NULL, NULL, 0, NULL},

  {"/إ(_H)", NULL, NULL, 0, "<LastBranch>"},
  {"/إ(_H)/(_A)", NULL, NULL, 0, NULL}
};

static int nmenu_items = sizeof(menu_items)/sizeof(menu_items[0]);
GtkItemFactory *factory = NULL;
GtkAccelGroup *accel_group = NULL;

void on_title_pic_toggled(gpointer data, gpointer action, GtkWidget* widget) {
  g_return_if_fail(widget != NULL);
  g_return_if_fail(monaicinfo != NULL);
  g_return_if_fail(picarea != NULL);
  g_return_if_fail(picarea_parent != NULL);

  if( GTK_CHECK_MENU_ITEM(widget)->active )
    monaicinfo->is_title_pic_showed = TRUE;
  else
    monaicinfo->is_title_pic_showed = FALSE;

  if( monaicinfo->is_title_pic_showed == FALSE )
    gtk_container_remove(GTK_CONTAINER(picarea_parent), picarea);
  else {
    picarea = gtk_drawing_area_new();
    monaicinfo->picarea = picarea;
    gtk_widget_set_usize(picarea,
      gdk_pixbuf_get_width(monaicinfo->boardinfo->pic),
      gdk_pixbuf_get_height(monaicinfo->boardinfo->pic));
    gtk_signal_connect(GTK_OBJECT(picarea), "expose_event",
                       (GtkSignalFunc)on_picarea_expose, NULL);
    gtk_widget_show(picarea);
    gtk_container_add(GTK_CONTAINER(picarea_parent), picarea);
  }

}

void on_picarea_expose(GtkWidget* widget, GdkEventExpose *event, gpointer data) {
  GdkPixbuf* pixbuf = NULL;
  int iw,ih;

  g_return_if_fail(widget != NULL);

  g_return_if_fail(monaicinfo->boardinfo != NULL);
  g_return_if_fail(monaicinfo->boardinfo->pic != NULL);

  pixbuf = monaicinfo->boardinfo->pic;

  iw = gdk_pixbuf_get_width(pixbuf);
  ih = gdk_pixbuf_get_height(pixbuf);

  if( gdk_pixbuf_get_has_alpha(pixbuf) )
    gdk_pixbuf_render_to_drawable_alpha(pixbuf, widget->window,
      0, 0, 0, 0, iw, ih, GDK_RGB_DITHER_NORMAL,
      GDK_PIXBUF_ALPHA_FULL, 0, 0, 0);
  else
    gdk_pixbuf_render_to_drawable(pixbuf, widget->window,
      widget->style->black_gc,
      0, 0, 0, 0, iw, ih, GDK_RGB_DITHER_NORMAL, 0, 0);
}

void get_board_pic(BoardInfo* boardinfo) {
  FILE* fp;

  g_return_if_fail(boardinfo != NULL);
  g_return_if_fail(boardinfo->bbs_title_picture != NULL);
  g_return_if_fail(dotpath != NULL);
  g_return_if_fail(dotpath->dir != NULL);


  dotpath->pic = g_strconcat(dotpath->dir, "/tmp.pic", NULL);
  if( (fp = fopen(dotpath->pic, "w")) == NULL ) {
    g_print("ե뤬񤭹ޤ: %s\n", dotpath->pic);
    exit(0);
  }
  mona_url2stream(boardinfo->bbs_title_picture, fp);
  fclose(fp);

  boardinfo->pic = gdk_pixbuf_new_from_file(dotpath->pic);
  if( picarea != NULL ) {
    gtk_widget_set_usize(GTK_WIDGET(picarea),
      gdk_pixbuf_get_width(boardinfo->pic),
      gdk_pixbuf_get_height(boardinfo->pic));
  }
}

void get_board_info(gchar* baseurl, BoardInfo* boardinfo) {
  gchar* tmpurl = NULL;
  gchar* tmpbuf = NULL;
  gchar* cursor = NULL;
  mona_http* tmphttp = NULL;
  gchar** linebufv = NULL;
  gchar** keyvalv = NULL;
  gint i;

  g_return_if_fail(baseurl != NULL);
  g_return_if_fail(boardinfo != NULL);

  tmpurl = g_strconcat(baseurl, "/SETTING.TXT", NULL);
  tmphttp = mona_http_get(tmpurl);

  g_return_if_fail(tmphttp != NULL);

  boardinfo->baseurl = g_strdup(baseurl);
  tmpbuf = sjis2euc(tmphttp->dat, tmphttp->size);

  boardinfo->ht = g_hash_table_new(g_str_hash, g_str_equal);
  linebufv = g_strsplit(tmpbuf, "\n", -1);
  g_assert(linebufv != NULL);
  for( i=0; linebufv[i] != NULL ; i++ ) {
    cursor = linebufv[i];
    g_assert(cursor != NULL);
    if( strstr(cursor, "=") == NULL )
      continue;
    keyvalv = g_strsplit(cursor, "=", -1);
    g_hash_table_insert(boardinfo->ht,
                        g_strdup(keyvalv[0]), g_strdup(keyvalv[1]));
    g_strfreev(keyvalv); 
  }

  boardinfo->bbs_title_picture =
    g_hash_table_lookup(boardinfo->ht, "BBS_TITLE_PICTURE");
  g_assert(boardinfo->bbs_title_picture != NULL);

  boardinfo->thlist = g_slist_alloc();
  boardinfo->datkey = g_slist_alloc();
  {
    gchar** eucdat = NULL;
    gchar** sbjdata = NULL;
    eucdat = mona_url2euc(g_strconcat(baseurl, "/subject.txt", NULL));
    while( ( cursor = *eucdat ) != NULL ) {
      sbjdata = g_strsplit(cursor, DELIMIT, -1);
      g_slist_append(boardinfo->datkey, sbjdata[0]);
      g_slist_append(boardinfo->thlist, sbjdata[1]);
      eucdat++;
    }
  }

  get_board_pic(monaicinfo->boardinfo);

  g_strfreev(linebufv);
  g_free(tmpbuf);
  g_free(tmpurl);
  g_free(tmphttp);
}

void monaic_quit() {
  gtk_main_quit();
}

void delete_all_children(GtkWidget* container) {
  GList* children = NULL;
  gint i;

  if( !GTK_IS_CONTAINER(container) )
    return;

  if( (children = gtk_container_children(GTK_CONTAINER(container))) == NULL )
    return;

  for(i=0; i<g_list_length(children); i++ ) {
    GtkWidget* widget;
    if( GTK_IS_CONTAINER(g_list_nth_data(children, i))) {
      widget = GTK_WIDGET(g_list_nth_data(children, i));
      delete_all_children(widget);
      gtk_container_remove(GTK_CONTAINER(container), widget);
      if ( GTK_IS_WIDGET(widget) && GTK_IS_OBJECT(widget) )
        gtk_widget_destroy(widget);
    } else if( GTK_IS_WIDGET(g_list_nth_data(children, i))) {
      widget = GTK_WIDGET(g_list_nth_data(children, i));
      gtk_container_remove(GTK_CONTAINER(container), widget);
      if ( GTK_IS_WIDGET(widget) && GTK_IS_OBJECT(widget) )
        gtk_widget_destroy(widget);
    }
  }
}

/* shrink space function:
   arg: "aaa    bbb" -> return: "aaa bbb"
*/
gchar* shrink_spaces(gchar* src) {
  gchar* result;
  gchar* resultptr;
  gint prevspace = 0;

  if ( src == NULL ) return NULL;

  resultptr = result = g_new0(gchar, strlen(src));
  for( ;*src != '\0'; src++ ) {
    if( prevspace == 0 && *(src) == ' ' ) {
      prevspace = 1;
    } else if ( prevspace == 1 && *(src) == ' ' ) {
      continue;
    } else if ( prevspace == 1 && *(src) != ' ' ) {
      prevspace = 0;
    }
    *(result++) = *(src);
  }

  return resultptr;
}

gchar* replace_str(gchar* src, gchar* fromstr, gchar* tostr) {
  if( src == NULL ) return NULL;
  if( fromstr == NULL ) return src;
  if( tostr == NULL ) tostr = g_strdup("");
  if( !strcmp(fromstr, tostr) ) {
    return src;
  }

  return g_strjoinv(tostr, g_strsplit(src, fromstr, -1));
}

article_dat* get_article_dat(gchar* buf) {
  gchar** strdptr = NULL;
  article_dat *result;
  gchar* article = NULL;

  if( buf == NULL ) return NULL;

  strdptr = g_strsplit(buf, DELIMIT, -1);

  if( strdptr == NULL ) return NULL;

  result = g_new0(article_dat, 1);
  result->name = strdptr[0];
  result->email = strdptr[1];
  result->date = strdptr[2];
  article = strdptr[3];
  article = shrink_spaces(article);
//  article = replace_str(article, "<br>", "\n");
//  article = replace_str(article, "&quot;", "\"");
//  article = replace_str(article, "", DELIMIT);
//  article = replace_str(article, "&gt;", ">");
//  article = replace_str(article, "&lt;", "<");
  result->article = g_strconcat(" ", article, NULL);

  return result;
}

gchar* get_text_from_article_dat(article_dat* adat) {
  if ( adat == NULL ) return NULL;

  return g_strconcat("̾: ", adat->name, " ",
                     ": ", adat->date, " \n",
                     " ", adat->article, "\n", NULL);
}

GSList* get_dat_from_key(gchar* key) {
  GSList* result = NULL;
  gchar** eucdat;
  gchar* cursor = NULL;
  gint limit = 10000;

  result = g_slist_alloc();
  eucdat = mona_url2euc(
   g_strconcat(monaicinfo->boardinfo->baseurl, "/dat/", key, NULL));
  while( (cursor = *eucdat) != NULL && limit > 0) {
    g_slist_append(result, get_article_dat(cursor));
    eucdat++;
    limit--;
  }

  return result;
}


void show_thread_article(gchar* key, gchar* thname) {
  //static GtkWidget* window = NULL;
  static GtkWidget* scrolled_win = NULL;
  static GtkWidget* monatext = NULL;


  if( scrolled_win == NULL ) {
    scrolled_win = gtk_scrolled_window_new(NULL, NULL);
    //gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_win),
    //                               GTK_POLICY_AUTOMATIC,
    //                               GTK_POLICY_AUTOMATIC);
    gtk_paned_add2(GTK_PANED(hpaned), scrolled_win);

    monatext = mona_text_new();
    gtk_container_add(GTK_CONTAINER(scrolled_win), monatext);
  }

  mona_text_set_article_slist(MONA_TEXT(monatext), get_dat_from_key(key));
  //gtk_widget_queue_draw(monatext);

  //gtk_window_set_title(GTK_WINDOW(window), thname);
  //gtk_widget_show_all(window);
  gtk_widget_show_all(hpaned);
}

void cb_itemsignal(GtkWidget *item, gchar *signame ) {
  gchar* name;
  GtkWidget* label;
  gint position;

  label = GTK_BIN(item)->child;
  gtk_label_get(GTK_LABEL(label), &name);
  if( GTK_IS_TREE(item->parent) ) {
    position = gtk_tree_child_position(GTK_TREE(item->parent), item)+1;
    show_thread_article(
      g_slist_nth_data(monaicinfo->boardinfo->datkey, position), name);
  }
}

char* get_thtitle_str(char* rawstr) {
  wchar_t wcstr[1024];
  wchar_t wcstr2[1024];
  char rawstr2[1024];
  int i,j;
  int intag = 0;
  int convlen = 0;
  int inwspace = 0; /* previous is white space */

  return rawstr;

  bzero(wcstr, sizeof(wcstr));
  bzero(wcstr2, sizeof(wcstr2));
  bzero(rawstr2, sizeof(rawstr2));
  convlen = mbstowcs(wcstr, rawstr, 1024);
  j=0;
  for(i=0; i<convlen; i++ ) {
    if ( wcstr[i] == ' ' || wcstr[i] == '\n' ) {
      if( inwspace == 0 ) {
        inwspace = 1;
        wcstr2[j++] = ' ';
        continue;
      } else if ( inwspace == 1 ) {
        continue;
      }
    } else
      inwspace = 0;

    if( wcstr[i] == '<' && intag == 0 ) {
      intag = 1;
    } else if( wcstr[i] == '>' && intag == 1 ) {
      intag = 0;
    } else if( intag == 0 ) {
      wcstr2[j++] = wcstr[i];
    }
  }
  wcstombs(rawstr2, wcstr2, 1024);
  return g_strdup(rawstr2);
}

MonaicInfo* monaic_get_monaicinfo() {
  g_assert(monaicinfo != NULL);
  return monaicinfo;
}

void monaic_gui_thread_update() {
  gint i;
  GtkWidget* subtree;
  GtkWidget* item;
  g_assert(monaicinfo != NULL);
  gtk_container_remove(GTK_CONTAINER(monaicinfo->maintree),
    monaicinfo->mainsub);

  /* FIXME: free & destroy old mainsub branches here */
  monaicinfo->mainsub = gtk_tree_item_new_with_label(
           g_hash_table_lookup(monaicinfo->boardinfo->ht, "BBS_TITLE"));
  gtk_tree_append(GTK_TREE(monaicinfo->maintree), monaicinfo->mainsub);

  subtree = gtk_tree_new();
  for( i=1; i<g_slist_length(monaicinfo->boardinfo->thlist); i++ ) {
    item = gtk_tree_item_new_with_label(g_strdup_printf("%d %s", i,
      get_thtitle_str(g_slist_nth_data(monaicinfo->boardinfo->thlist, i))));
    gtk_signal_connect(GTK_OBJECT(item), "select",
                       GTK_SIGNAL_FUNC(cb_itemsignal), "select");
    gtk_widget_show(item);
    gtk_tree_append(GTK_TREE(subtree), item);
  }

  gtk_tree_item_set_subtree(GTK_TREE_ITEM(monaicinfo->mainsub), subtree);
  gtk_tree_item_expand(GTK_TREE_ITEM(monaicinfo->mainsub));
  gtk_widget_show_all(monaicinfo->maintree);
}

int main(int argc, char** argv) {
  GtkWidget* window;
  GtkWidget* handlebox;
  GtkWidget* menubar;
  GtkWidget* vbox;
  GtkWidget* scrolled_win;
  GtkWidget* tree;
  GtkWidget* subtree;
  GtkWidget* item;
  GtkWidget* tmpitem;
  gint i;

  dotpath = g_new0(DotPath, 1);
  monaicinfo = g_new0(MonaicInfo, 1);
  monaicinfo->is_title_pic_showed = TRUE;

  if( getenv("HOME") == NULL ) {
    g_print("ۡǥ쥯ȥ꤬Ǥޤ\n");
    exit(0);
  }
  dotpath->dir = g_strconcat(getenv("HOME"), "/.monaic", NULL);
  if( mkdir(dotpath->dir, 0755) == -1 ) {
    if( errno != EEXIST ) {
      g_print("~/.monaic ǥ쥯ȥǤޤ\n");
      exit(0);
    }
  }

  monaicinfo->boardinfo = g_new0(BoardInfo, 1);
  get_board_info(BASEURL, monaicinfo->boardinfo);

  gtk_set_locale();
  gtk_init(&argc, &argv);
  gdk_rgb_init();

  gtk_rc_parse("/usr/share/monaic/monaic.gtkrc");

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_signal_connect(GTK_OBJECT(window), "delete_event",
                     GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
  gtk_window_set_title(GTK_WINDOW(window), "N.I.C.H. Monaic");
  gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, TRUE);

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(window), vbox);
  handlebox = gtk_handle_box_new();
  gtk_container_add(GTK_CONTAINER(vbox), handlebox);

  picarea = gtk_drawing_area_new();
  monaicinfo->picarea = picarea;
  gtk_widget_set_usize(picarea,
    gdk_pixbuf_get_width(monaicinfo->boardinfo->pic),
    gdk_pixbuf_get_height(monaicinfo->boardinfo->pic));
  gtk_signal_connect(GTK_OBJECT(picarea), "expose_event",
                     (GtkSignalFunc)on_picarea_expose, NULL);
  picarea_parent = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(vbox), picarea_parent);

  accel_group = gtk_accel_group_new();
  factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<Main>", accel_group);
  //gtk_item_factory_set_translate_func(factory, menu_translate, NULL, NULL);

  gtk_item_factory_create_items(factory, nmenu_items, menu_items, window);
  gtk_accel_group_attach(accel_group, GTK_OBJECT(window));

  menubar = gtk_item_factory_get_widget(factory, "<Main>");

  gtk_container_add(GTK_CONTAINER(handlebox), menubar);
  tmpitem = gtk_item_factory_get_item(factory, "/ɽ(V)/ĥХʡ(P)");
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(tmpitem), TRUE);

  tmpitem = gtk_item_factory_get_item(factory, "/å(T)");
  add_bbs_table(GTK_MENU_ITEM(tmpitem));

  hpaned = gtk_hpaned_new();
  gtk_container_add(GTK_CONTAINER(vbox), hpaned);
  scrolled_win = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_win),
                                 GTK_POLICY_AUTOMATIC,
                                 GTK_POLICY_AUTOMATIC);
  gtk_widget_set_usize(scrolled_win, 200, 400);
  gtk_paned_add1(GTK_PANED(hpaned), scrolled_win);

  tree = gtk_tree_new();
  monaicinfo->maintree = tree;
  g_assert(monaicinfo->boardinfo != NULL);
  item = gtk_tree_item_new_with_label(
           g_hash_table_lookup(monaicinfo->boardinfo->ht, "BBS_TITLE"));
  gtk_tree_append(GTK_TREE(tree), item);
  monaicinfo->mainsub = item;
  subtree = gtk_tree_new();
  gtk_tree_set_view_mode(GTK_TREE(subtree), GTK_TREE_VIEW_ITEM);
  gtk_tree_item_set_subtree(GTK_TREE_ITEM(item), subtree);

  for( i=1; i<g_slist_length(monaicinfo->boardinfo->thlist); i++ ) {
    item = gtk_tree_item_new_with_label(g_strdup_printf("%d %s", i,
      get_thtitle_str(g_slist_nth_data(monaicinfo->boardinfo->thlist, i))));
    gtk_signal_connect(GTK_OBJECT(item), "select",
                       GTK_SIGNAL_FUNC(cb_itemsignal), "select");
    gtk_widget_show(item);
    gtk_tree_append(GTK_TREE(subtree), item);
  }
  gtk_tree_item_expand(GTK_TREE_ITEM(monaicinfo->mainsub));
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_win),
                                   tree);
  gtk_widget_show_all(window);
  gtk_main();
  return 0;
}
