/*
 * N.I.C.H. Monaic - The 2 channel bbs viewer
 *
 * Copyright (C) 2001 Free Software Foundation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */

/*
 * See the AUTHORS file for a list of people on the Monaic Team. See
 * the ChangeLog files for a list of changes. These files are distributed
 * with Monaic at http://monaic.sourceforge.net/
 */

#ifndef __MONA_TEXT_H
#define __MONA_TEXT_H

#include <gtk/gtk.h>

#ifdef __cplusplus
extern "C" {
#endif

#define MONA_TEXT(obj) \
    GTK_CHECK_CAST(obj, mona_text_get_type(), MonaText)
#define MONA_TEXT_CLASS(klass) \
    GTK_CHECK_CLASS_CAST(klass, mona_text_get_type(), MonaTextClass)
#define MONA_IS_TEXT(obj) \
    GTK_CHECK_TYPE(obj, mona_text_get_type())

typedef struct _MonaText MonaText;
typedef struct _MonaTextClass MonaTextClass;
typedef struct _PosData PosData;
typedef struct _PosPair PosPair;

typedef enum {
  MONA_LINK_URL,
  MONA_LINK_EMAIL,
  MONA_LINK_ARTICLE
} MonaLinkType;

struct _MonaText {
  GtkWidget widget;
  GSList* article_slist;
  GSList* pos_cache;
  guint offset_y;
  guint total_height;

  PosPair* active_pospair;

  GtkAdjustment* hadj;
  GtkAdjustment* vadj;
};

struct _MonaTextClass {
  GtkWidgetClass parent_class;

  void  (*set_scroll_adjustments)   (MonaText        *text,
                                     GtkAdjustment  *hadjustment,
                                     GtkAdjustment  *vadjustment);
};

typedef struct _article_dat {
  gchar* name;
  gchar* email;
  gchar* date;
  gchar* article;
} article_dat;

struct _PosPair {
  GdkRectangle* rect;
  gchar* text;
  MonaLinkType type;
  gchar* url;
};

struct _PosData {
  guint start;
  GSList* pair_slist;
};

GtkWidget* mona_text_new();
void mona_text_set_article_slist(MonaText* text, GSList* article_slist);
guint mona_text_get_type(void);

#ifdef __cplusplus
}
#endif

#endif /* __MONA_TEXT_H */
